# fmt: off
import pytest

from streamsets.sdk.constants import StageConfigurationProperty
from streamsets.sdk.sch_models import Connection, SchSdcStage, SchStStage

# fmt: on

NUM_OF_STAGES = 5


class DummyPipeline:
    """
    A Dummy Pipeline Class
    """

    def __init__(self):
        self.stages = []

    def add_stage(self, *stages):
        for stage in stages:
            stage._pipeline = self
            self.stages.append(stage)


class DummyPipelineBuilder:
    """
    A Dummy Pipeline Class Builder
    """

    def __init__(self):
        self._config_key = 'pipelineBuilder'
        self._pipeline = {self._config_key: {'stages': []}}

    def add_stage(self, *stages):
        for stage in stages:
            stage._pipeline = self
            self._pipeline[self._config_key]['stages'].append(stage._data)


@pytest.fixture(scope="function")
def dummy_pipeline():
    return DummyPipeline()


@pytest.fixture(scope="function")
def dummy_pipeline_builder():
    return DummyPipelineBuilder()


@pytest.fixture(scope="function")
def dev_data_generator_sdc_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'DevDataGenerator_{}'.format(i),
            'library': 'streamsets-datacollector-dev-lib',
            'stageName': 'com_streamsets_pipeline_stage_devtest_RandomDataGeneratorSource',
            'stageVersion': '6',
            'configuration': [
                {'name': 'delay', 'value': 1000},
                {'name': 'numberOfRecords', 'value': 0},
                {'name': 'eventName', 'value': 'generated-event'},
                {'name': 'batchSize', 'value': 1000},
                {'name': 'headerAttributes', 'value': []},
                {
                    'name': 'dataGenConfigs',
                    'value': [{'type': 'STRING', 'precision': 10, 'scale': 2, 'fieldAttributes': []}],
                },
                {'name': 'numThreads', 'value': 1},
                {'name': 'rootFieldType', 'value': 'MAP'},
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
            ],
            'uiInfo': {
                'colorIcon': 'Origin_Dev_Data_Generator.png',
                'outputStreamLabels': None,
                'yPos': 50,
                'stageType': 'SOURCE',
                'rawSource': None,
                'icon': 'dev.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Dev Data Generator {}'.format(i),
                'xPos': 60,
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['DevDataGenerator_1OutputLane1694131201219{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchSdcStage(stage=stage_json))

    return stages


@pytest.fixture(scope="function")
def dev_data_generator_st_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'DevDataGenerator_{}'.format(i),
            'library': 'streamsets-datacollector-dev-lib',
            'stageName': 'com_streamsets_pipeline_stage_devtest_RandomDataGeneratorSource',
            'stageVersion': '6',
            'configuration': [
                {'name': 'delay', 'value': 1000},
                {'name': 'numberOfRecords', 'value': 0},
                {'name': 'eventName', 'value': 'generated-event'},
                {'name': 'batchSize', 'value': 1000},
                {'name': 'headerAttributes', 'value': []},
                {
                    'name': 'dataGenConfigs',
                    'value': [{'type': 'STRING', 'precision': 10, 'scale': 2, 'fieldAttributes': []}],
                },
                {'name': 'numThreads', 'value': 1},
                {'name': 'rootFieldType', 'value': 'MAP'},
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
            ],
            'uiInfo': {
                'colorIcon': 'Origin_Dev_Data_Generator.png',
                'outputStreamLabels': None,
                'yPos': 50,
                'stageType': 'SOURCE',
                'rawSource': None,
                'icon': 'dev.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Dev Data Generator {}'.format(i),
                'xPos': 60,
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['DevDataGenerator_1OutputLane1694131201219{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchStStage(stage=stage_json))

    return stages


@pytest.fixture(scope="function")
def trash_sdc_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'Trash_{}'.format(i),
            'library': 'streamsets-datacollector-basic-lib',
            'stageName': 'com_streamsets_pipeline_stage_destination_devnull_NullDTarget',
            'stageVersion': '1',
            'configuration': [],
            'uiInfo': {
                'colorIcon': 'Destination_Trash.png',
                'outputStreamLabels': None,
                'yPos': 180,
                'stageType': 'TARGET',
                'rawSource': None,
                'icon': 'trash.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Trash {}'.format(i),
                'xPos': 940,
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': [],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchSdcStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def trash_st_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'Trash_{}'.format(i),
            'library': 'streamsets-datacollector-basic-lib',
            'stageName': 'com_streamsets_pipeline_stage_destination_devnull_NullDTarget',
            'stageVersion': '1',
            'configuration': [],
            'uiInfo': {
                'colorIcon': 'Destination_Trash.png',
                'outputStreamLabels': None,
                'yPos': 180,
                'stageType': 'TARGET',
                'rawSource': None,
                'icon': 'trash.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Trash {}'.format(i),
                'xPos': 940,
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': [],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchStStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def delay_sdc_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'Delay_{}'.format(i),
            'library': 'streamsets-datacollector-basic-lib',
            'stageName': 'com_streamsets_pipeline_stage_processor_delay_DelayProcessor',
            'stageVersion': '2',
            'configuration': [
                {'name': 'delay', 'value': 1000},
                {'name': 'skipDelayOnEmptyBatch', 'value': False},
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
                {'name': 'stageRequiredFields', 'value': []},
                {'name': 'stageRecordPreconditions', 'value': []},
            ],
            'uiInfo': {
                'colorIcon': 'Processor_Delay.png',
                'outputStreamLabels': None,
                'yPos': '233.043875',
                'stageType': 'PROCESSOR',
                'rawSource': None,
                'icon': 'delay.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Delay {}'.format(i),
                'xPos': '1072.1999999999998',
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['Delay_1OutputLane1694191364195{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchSdcStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def delay_st_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'Delay_{}'.format(i),
            'library': 'streamsets-datacollector-basic-lib',
            'stageName': 'com_streamsets_pipeline_stage_processor_delay_DelayProcessor',
            'stageVersion': '2',
            'configuration': [
                {'name': 'delay', 'value': 1000},
                {'name': 'skipDelayOnEmptyBatch', 'value': False},
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
                {'name': 'stageRequiredFields', 'value': []},
                {'name': 'stageRecordPreconditions', 'value': []},
            ],
            'uiInfo': {
                'colorIcon': 'Processor_Delay.png',
                'outputStreamLabels': None,
                'yPos': '233.043875',
                'stageType': 'PROCESSOR',
                'rawSource': None,
                'icon': 'delay.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Delay {}'.format(i),
                'xPos': '1072.1999999999998',
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['Delay_1OutputLane1694191364195{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchStStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def dev_identity_sdc_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'DevIdentity_{}'.format(i),
            'library': 'streamsets-datacollector-dev-lib',
            'stageName': 'com_streamsets_pipeline_stage_processor_identity_IdentityProcessor',
            'stageVersion': '1',
            'configuration': [
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
                {'name': 'stageRequiredFields', 'value': []},
                {'name': 'stageRecordPreconditions', 'value': []},
            ],
            'uiInfo': {
                'colorIcon': 'Processor_Dev_Identity.png',
                'outputStreamLabels': None,
                'yPos': '387.065875',
                'stageType': 'PROCESSOR',
                'rawSource': None,
                'icon': 'dev.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Dev Identity {}'.format(i),
                'xPos': '1083.1999999999998',
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['DevIdentity_1OutputLane1694195672155{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchSdcStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def dev_identity_st_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'DevIdentity_{}'.format(i),
            'library': 'streamsets-datacollector-dev-lib',
            'stageName': 'com_streamsets_pipeline_stage_processor_identity_IdentityProcessor',
            'stageVersion': '1',
            'configuration': [
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
                {'name': 'stageRequiredFields', 'value': []},
                {'name': 'stageRecordPreconditions', 'value': []},
            ],
            'uiInfo': {
                'colorIcon': 'Processor_Dev_Identity.png',
                'outputStreamLabels': None,
                'yPos': '387.065875',
                'stageType': 'PROCESSOR',
                'rawSource': None,
                'icon': 'dev.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Dev Identity {}'.format(i),
                'xPos': '1083.1999999999998',
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': ['DevIdentity_1OutputLane1694195672155{}'.format(i)],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchStStage(stage=stage_json))
    return stages


@pytest.fixture(scope="function")
def record_dedup_sdc_stage():
    stages = []
    for i in range(1, NUM_OF_STAGES + 1):
        stage_json = {
            'instanceName': 'RecordDeduplicator_{}'.format(i),
            'library': 'streamsets-datacollector-basic-lib',
            'stageName': 'com_streamsets_pipeline_stage_processor_dedup_DeDupDProcessor',
            'stageVersion': '1',
            'configuration': [
                {'name': 'fieldsToCompare', 'value': []},
                {'name': 'compareFields', 'value': 'ALL_FIELDS'},
                {'name': 'timeWindowSecs', 'value': 0},
                {'name': 'recordCountWindow', 'value': 1000000},
                {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
                {'name': 'stageRequiredFields', 'value': []},
                {'name': 'stageRecordPreconditions', 'value': []},
            ],
            'uiInfo': {
                'colorIcon': 'Processor_Record_Deduplicator.png',
                'outputStreamLabels': ['Unique Records', 'Duplicate Records'],
                'yPos': '253.065875',
                'stageType': 'PROCESSOR',
                'rawSource': None,
                'icon': 'dedup.png',
                'description': '',
                'inputStreamLabels': None,
                'label': 'Record Deduplicator {}'.format(i),
                'xPos': '1196.1999999999998',
                'displayMode': 'BASIC',
            },
            'inputLanes': [],
            'outputLanes': [
                'RecordDeduplicator_1OutputLane169419698035{}0'.format(i),
                'RecordDeduplicator_1OutputLane169419698035{}1'.format(i),
            ],
            'eventLanes': [],
            'services': [],
        }
        stages.append(SchSdcStage(stage=stage_json))
    return stages


@pytest.mark.parametrize(
    "origin_stage, destination_stage, processor_stage_one, processor_stage_two",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage", "dev_identity_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage", "dev_identity_st_stage"),
    ],
)
def test_copy_inputs_sanity(origin_stage, destination_stage, processor_stage_one, processor_stage_two, request):
    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage_one = request.getfixturevalue(processor_stage_one)[0]
    processor_stage_two = request.getfixturevalue(processor_stage_two)[0]

    origin_stage >> processor_stage_one >> destination_stage
    origin_stage >> processor_stage_two

    destination_stage_input_lanes = destination_stage._data['inputLanes'][:]
    processor_stage_two_input_lanes = processor_stage_two._data['inputLanes'][:]
    assert destination_stage._data['inputLanes'] == destination_stage_input_lanes
    assert processor_stage_two._data['inputLanes'] == processor_stage_two_input_lanes

    processor_stage_two.copy_inputs(destination_stage)
    assert processor_stage_two._data['inputLanes'] == processor_stage_two_input_lanes + destination_stage_input_lanes


@pytest.mark.parametrize(
    "origin_stage,destination_stage,processor_stage_one,processor_stage_two",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage", "dev_identity_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage", "dev_identity_st_stage"),
    ],
)
def test_copy_inputs_override_true(origin_stage, destination_stage, processor_stage_one, processor_stage_two, request):
    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage_one = request.getfixturevalue(processor_stage_one)[0]
    processor_stage_two = request.getfixturevalue(processor_stage_two)[0]

    origin_stage >> processor_stage_one >> destination_stage
    origin_stage >> processor_stage_two

    destination_stage_input_lanes = destination_stage._data['inputLanes'][:]
    processor_stage_two_input_lanes = processor_stage_two._data['inputLanes'][:]
    assert destination_stage._data['inputLanes'] == destination_stage_input_lanes
    assert processor_stage_two._data['inputLanes'] == processor_stage_two_input_lanes

    processor_stage_two.copy_inputs(destination_stage, override=True)
    assert processor_stage_two._data['inputLanes'] == destination_stage_input_lanes


@pytest.mark.parametrize(
    "origin_stage, destination_stage, processor_stage",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage"),
    ],
)
def test_copy_outputs_from_pipeline_stages_sanity(origin_stage, destination_stage, processor_stage, request):
    pipeline = DummyPipeline()

    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage = request.getfixturevalue(processor_stage)[0]

    pipeline.add_stage(origin_stage, destination_stage, processor_stage)

    origin_stage >> destination_stage

    assert origin_stage._data['outputLanes'][0] in destination_stage._data['inputLanes']
    assert processor_stage._data['outputLanes'][0] not in destination_stage._data['inputLanes']

    processor_stage.copy_outputs(origin_stage)
    assert processor_stage._data['outputLanes'][0] in destination_stage._data['inputLanes']
    assert origin_stage._data['outputLanes'][0] in destination_stage._data['inputLanes']


def test_copy_outputs_from_pipeline_stages_invalid_num_output_lanes(
    dev_data_generator_sdc_stage, trash_sdc_stage, delay_sdc_stage, record_dedup_sdc_stage
):
    pipeline = DummyPipeline()

    origin_sdc_stage = dev_data_generator_sdc_stage[0]
    destination_sdc_stage = trash_sdc_stage[0]
    processor_sdc_stage = delay_sdc_stage[0]
    multi_output_lane_processor_sdc_stage = record_dedup_sdc_stage[0]

    pipeline.add_stage(
        origin_sdc_stage, destination_sdc_stage, processor_sdc_stage, multi_output_lane_processor_sdc_stage
    )
    origin_sdc_stage >> processor_sdc_stage >> destination_sdc_stage

    with pytest.raises(ValueError):
        multi_output_lane_processor_sdc_stage.copy_outputs(processor_sdc_stage)


def test_copy_outputs_from_pipeline_stages_multiple_output_lane_stage(
    dev_data_generator_sdc_stage, trash_sdc_stage, delay_sdc_stage, record_dedup_sdc_stage, dummy_pipeline
):
    dev = dev_data_generator_sdc_stage[0]
    trash1 = trash_sdc_stage[0]
    trash2 = trash_sdc_stage[1]
    dedup1 = record_dedup_sdc_stage[0]
    dedup2 = record_dedup_sdc_stage[1]

    dummy_pipeline.add_stage(dev, trash1, trash2, dedup1, dedup2)

    dev >> dedup1 >> trash1
    dedup1 >> trash2

    assert dedup1._data['outputLanes'][0] == trash1._data['inputLanes'][0]
    assert dedup1._data['outputLanes'][1] == trash2._data['inputLanes'][0]

    dedup2.copy_outputs(dedup1)

    assert dedup1._data['outputLanes'][0] == trash1._data['inputLanes'][0]
    assert dedup1._data['outputLanes'][1] == trash2._data['inputLanes'][0]
    assert dedup2._data['outputLanes'][0] == trash1._data['inputLanes'][1]
    assert dedup2._data['outputLanes'][1] == trash2._data['inputLanes'][1]


@pytest.mark.parametrize(
    "origin_stage,destination_stage,processor_stage",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage"),
    ],
)
def test_copy_outputs_from_stages_in_pipeline_builder_sanity(
    origin_stage, destination_stage, processor_stage, dummy_pipeline_builder, request
):
    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage = request.getfixturevalue(processor_stage)[0]

    dummy_pipeline_builder.add_stage(origin_stage, destination_stage, processor_stage)
    origin_stage >> destination_stage

    assert processor_stage._data['outputLanes'][0] not in destination_stage._data['inputLanes']
    processor_stage.copy_outputs(origin_stage)
    assert processor_stage._data['outputLanes'][0] in destination_stage._data['inputLanes']
    assert origin_stage._data['outputLanes'][0] in destination_stage._data['inputLanes']


@pytest.mark.parametrize(
    "origin_stage, destination_stage, processor_stage_one, processor_stage_two",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage", "dev_identity_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage", "dev_identity_st_stage"),
    ],
)
def test_copy_inputs_from_different_stage(
    origin_stage, destination_stage, processor_stage_one, processor_stage_two, request
):
    pipeline = DummyPipeline()
    pipeline2 = DummyPipeline()

    origin_stages = request.getfixturevalue(origin_stage)
    origin_stage, origin_stage_two = origin_stages[0], origin_stages[1]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage_one = request.getfixturevalue(processor_stage_one)[0]
    processor_stage_two = request.getfixturevalue(processor_stage_two)[0]

    pipeline.add_stage(origin_stage, destination_stage, processor_stage_one)
    pipeline2.add_stage(origin_stage_two, processor_stage_two)

    origin_stage >> processor_stage_one >> destination_stage
    origin_stage_two >> processor_stage_two

    with pytest.raises(ValueError):
        processor_stage_two.copy_inputs(destination_stage)


@pytest.mark.parametrize(
    "origin_stage, destination_stage, processor_stage",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage", "delay_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage", "delay_st_stage"),
    ],
)
def test_copy_outputs_from_different_stage(origin_stage, destination_stage, processor_stage, request):
    pipeline = DummyPipeline()
    pipeline2 = DummyPipeline()

    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]
    processor_stage = request.getfixturevalue(processor_stage)[0]

    pipeline.add_stage(origin_stage, destination_stage)
    pipeline2.add_stage(processor_stage)

    origin_stage >> destination_stage

    with pytest.raises(ValueError):
        processor_stage.copy_outputs(origin_stage)


@pytest.mark.parametrize("origin_stage", ["dev_data_generator_sdc_stage", "dev_data_generator_st_stage"])
def test_set_output_lanes(origin_stage, request):
    origin_stage = request.getfixturevalue(origin_stage)[0]

    assert origin_stage.output_lanes
    origin_stage_output_lanes = origin_stage.output_lanes

    # assert that origin_stage.output_lanes did not change
    origin_stage.output_lanes = ['dummy_value']
    assert origin_stage.output_lanes == origin_stage_output_lanes


@pytest.mark.parametrize(
    "origin_stage, destination_stage",
    [
        ("dev_data_generator_sdc_stage", "trash_sdc_stage"),
        ("dev_data_generator_st_stage", "trash_st_stage"),
    ],
)
def test_set_input_lanes(origin_stage, destination_stage, request):
    origin_stage = request.getfixturevalue(origin_stage)[0]
    destination_stage = request.getfixturevalue(destination_stage)[0]

    origin_stage.connect_outputs([destination_stage])

    assert destination_stage.input_lanes
    destination_stage_input_lanes = destination_stage.input_lanes

    # assert that destination_stage.input_lanes did not change
    destination_stage.input_lanes = ['dummy_value']
    assert destination_stage.input_lanes == destination_stage_input_lanes


@pytest.mark.parametrize(
    "dev_data_gen, trash",
    [("dev_data_generator_sdc_stage", "trash_sdc_stage"), ("dev_data_generator_st_stage", "trash_st_stage")],
)
def test_event_lane_sanity(dev_data_gen, trash, request):
    dev_data_gen = request.getfixturevalue(dev_data_gen)[0]
    trashes = request.getfixturevalue(trash)
    trash1, trash2 = trashes[0], trashes[1]

    # assert base case, no event_lanes exist
    assert len(dev_data_gen.event_lanes) == 0
    assert len(trash2.input_lanes) == 0

    dev_data_gen.connect_outputs(stages=[trash1], event_lane=False)
    dev_data_gen.connect_outputs(stages=[trash2], event_lane=True)

    # assert that event lane have been created
    assert len(dev_data_gen.event_lanes) == 1
    assert '_EventLane' in trash2.input_lanes[0]
    assert dev_data_gen.event_lanes == trash2.input_lanes


@pytest.mark.parametrize(
    "dev_data_gen, trash",
    [("dev_data_generator_sdc_stage", "trash_sdc_stage"), ("dev_data_generator_st_stage", "trash_st_stage")],
)
def test_only_one_event_lane_created_for_stage(dev_data_gen, trash, request):
    dev_data_gen = request.getfixturevalue(dev_data_gen)[0]
    trashes = request.getfixturevalue(trash)
    trash1, trash2, trash3 = trashes[0], trashes[1], trashes[2]

    # assert base case, no event_lanes exist
    assert len(dev_data_gen.event_lanes) == 0
    assert len(trash2.input_lanes) == 0
    assert len(trash3.input_lanes) == 0

    dev_data_gen.connect_outputs(stages=[trash1], event_lane=False)
    dev_data_gen.connect_outputs(stages=[trash2], event_lane=True)
    dev_data_gen.connect_outputs(stages=[trash3], event_lane=True)

    # assert that only one event lanes has been created
    assert len(dev_data_gen.event_lanes) == 1
    assert '_EventLane' in dev_data_gen.event_lanes[0]
    assert dev_data_gen.event_lanes == trash2.input_lanes
    assert dev_data_gen.event_lanes == trash3.input_lanes


def test_use_connection_sanity(mocker):
    mock_control_hub = mocker.Mock()
    stage_json = {
        'instanceName': 'foo',
        'library': 'streamsets-datacollector-foo-lib',
        'stageName': 'com_streamsets_foo_stage',
        'stageVersion': '2',
        'configuration': [{'name': 'connection', 'value': 'MANUAL'}],
        'inputLanes': [],
        'outputLanes': [],
        'eventLanes': [],
        'services': [],
        'uiInfo': {
            'colorIcon': 'Origin_Foo_Generator.png',
            'stageType': 'SOURCE',
        },
    }

    connection = Connection(
        connection={
            'connection': 'foo',
            'connectionType': 'bar',
            'id': 1,
            'connectionDefinition': '{"configuration": [{"name": "foo", "value": "bar"}]}',
        },
        control_hub=mock_control_hub,
    )
    stage_instance = type(
        "DummyStage",
        (SchSdcStage,),
        {'_attributes': {'connection': [StageConfigurationProperty(config_name='connection')]}},
    )

    stage = stage_instance(stage=stage_json, supported_connection_types=['bar'])

    assert stage.configuration['connection'] == 'MANUAL'
    assert stage.connection == 'MANUAL'

    stage.use_connection(connection)
    assert stage.connection == 1


def test_use_connection_unsupported_stage_type(mocker):
    mock_control_hub = mocker.Mock()
    stage_json = {
        'instanceName': 'foo',
        'library': 'streamsets-datacollector-foo-lib',
        'stageName': 'com_streamsets_foo_stage',
        'stageVersion': '2',
        'configuration': [],
        'inputLanes': [],
        'outputLanes': [],
        'eventLanes': [],
        'services': [],
        'uiInfo': {
            'colorIcon': 'Origin_Foo_Generator.png',
            'stageType': 'SOURCE',
        },
    }

    connection = Connection(
        connection={
            'connection': 'foo',
            'connectionType': 'bar',
            'id': 1,
            'connectionDefinition': '{"configuration": [{"name": "foo", "value": "bar"}]}',
        },
        control_hub=mock_control_hub,
    )
    stage = SchSdcStage(stage=stage_json)

    assert 'connection' not in stage.configuration
    assert not hasattr(stage, 'connection')

    with pytest.raises(ValueError):
        stage.use_connection(connection)


def test_use_connection_unsupported_connection_type(mocker):
    mock_control_hub = mocker.Mock()
    stage_json = {
        'instanceName': 'foo',
        'library': 'streamsets-datacollector-foo-lib',
        'stageName': 'com_streamsets_foo_stage',
        'stageVersion': '2',
        'configuration': [{'name': 'connection', 'value': 'MANUAL'}],
        'inputLanes': [],
        'outputLanes': [],
        'eventLanes': [],
        'services': [],
        'uiInfo': {
            'colorIcon': 'Origin_Foo_Generator.png',
            'stageType': 'SOURCE',
        },
    }

    connection = Connection(
        connection={
            'connection': 'foo',
            'connectionType': 'foo',
            'id': 1,
            'connectionDefinition': '{"configuration": [{"name": "foo", "value": "bar"}]}',
        },
        control_hub=mock_control_hub,
    )
    stage_instance = type(
        "DummyStage",
        (SchSdcStage,),
        {'_attributes': {'connection': [StageConfigurationProperty(config_name='connection')]}},
    )

    stage = stage_instance(stage=stage_json, supported_connection_types=['bar'])

    assert stage.configuration['connection'] == 'MANUAL'
    assert stage.connection == 'MANUAL'
    assert connection.connection_type not in stage.supported_connection_types

    with pytest.raises(ValueError):
        stage.use_connection(connection)



DUMMY_DEPLOYMENT_JSON = {
    'id': '1fd7acec-817f-4ee4-b0cc-7046a06744e7:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'organization': '35be715e-e8e9-11ec-8e84-93331d4150d4',
    'name': 'WOrds',
    'envId': '008c3d5b-49f4-44d8-ba40-c906854b3829:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'environment': '008c3d5b-49f4-44d8-ba40-c906854b3829:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'engineVersion': "5.9.0",
    'engineType': "DC",
    'engineConfiguration': {"engineType": "DC", "engineVersion": "5.9.0", "engineVersionId": "DC:5.10.0::RC3"},
    'type': 'SELF',
    'desiredInstances': 1,
    'scalaBinaryVersion': "2.12",
    'state': 'ENABLED',
    'status': 'OK',
    'stateDisplayLabel': 'ACTIVE',
    'statusDetail': '',
    'statusLastUpdated': 1702313113931,
    'creator': '2cfced39-e8e9-11ec-8e84-d7b86be914f5@35be715e-e8e9-11ec-8e84-93331d4150d4',
    'createTime': 1695922045569,
    'lastModifiedBy': '2cfced39-e8e9-11ec-8e84-d7b86be914f5@35be715e-e8e9-11ec-8e84-93331d4150d4',
    'lastModifiedOn': 1695922788328,
    'deploymentTags': None,
    'rawDeploymentTags': ['self-managed-tag'],
    'locked': False,
    'lockReference': None,
    'lockChangedAt': None,
    'lockChangedBy': None,
    'engineShutdownTimeout': 10,
    'engineShutdownTimeoutTimestamp': None,
    'showClonedConfigDiff': None,
    'installType': 'TARBALL',
}

DUMMY_DEPLOYMENT_JSON_2 = {
    'id': '1fd7acec-817f-4ee4-b0cc-7046a06744e7:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'organization': '35be715e-e8e9-11ec-8e84-93331d4150d4',
    'name': "Don't Update",
    'envId': '008c3d5b-49f4-44d8-ba40-c906854b3829:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'environment': '008c3d5b-49f4-44d8-ba40-c906854b3829:35be715e-e8e9-11ec-8e84-93331d4150d4',
    'engineVersion': "5.9.0",
    'engineType': "DC",
    'engineConfiguration': {
        "engineType": "DC",
        "engineVersion": "5.9.0",
        "engineVersionId": "DC:5.10.0::RC3",
        'jvmConfig': {"a": "a"},
        'stageLibs': None,
        'labels': None,
        'externalResourcesUri': None,
    },
    'type': 'SELF',
    'desiredInstances': 1,
    'scalaBinaryVersion': "2.12",
    'state': 'ENABLED',
    'status': 'OK',
    'stateDisplayLabel': 'ACTIVE',
    'statusDetail': '',
    'statusLastUpdated': 1702313113931,
    'creator': '2cfced39-e8e9-11ec-8e84-d7b86be914f5@35be715e-e8e9-11ec-8e84-93331d4150d4',
    'createTime': 1695922045569,
    'lastModifiedBy': '2cfced39-e8e9-11ec-8e84-d7b86be914f5@35be715e-e8e9-11ec-8e84-93331d4150d4',
    'lastModifiedOn': 1695922788328,
    'deploymentTags': None,
    'rawDeploymentTags': ['self-managed-tag'],
    'locked': False,
    'lockReference': None,
    'lockChangedAt': None,
    'lockChangedBy': None,
    'engineShutdownTimeout': 10,
    'engineShutdownTimeoutTimestamp': None,
    'showClonedConfigDiff': None,
    'installType': 'TARBALL',
}



# Copyright 2022 StreamSets Inc.

# fmt: off
import json

import pytest

from streamsets.sdk.utils import SeekableList, get_random_string

# fmt: on

STAGE_NAMES = ['Dev Data Generator', 'Expression Evaluator', 'Trash', 'Field Renamer']


@pytest.mark.parametrize(
    'permutation',
    (
        ['Origins', {'0': ['0']}],
        ['Processors', {'1': ['1']}],
        ['Destinations', {'2': ['2']}],
        ['Origins', {'0': ['1'], '1': ['1']}],
        ['Processors', {'1': ['3'], '3': ['3']}],
        ['Destinations', {'1': ['2'], '2': ['2']}],
        ['Origins', {'0': ['1', '3'], '1': ['1'], '3': ['3']}],
        ['Origins', {'0': ['1+0', '1+1'], '1+0': ['1+0'], '1+1': ['1+1']}],
        ['Destinations', {'0': ['2'], '1': ['2'], '2': ['2']}],
    ),
)
def test_create_fragments(sch, sch_authoring_sdc_id, permutation):
    """Each parameter represents a label and fragment graph. The numbers are the indices of the variable STAGE_NAMES.
    e.g. ['Origins', {'0': ['1+0', '1+1'], '1+0': ['1+0'], '1+1': ['1+1']}] represents a fragment:
            'Dev Data Generator' >> ['Expression Evaluator 0', 'Expression Evaluator 1']
    with label 'Origins'
    """
    pipeline_builder = sch.get_pipeline_builder(
        engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
    )
    stages = {}
    label, graph = permutation
    for node in graph:
        component = node
        if '+' in node:
            component = node.split('+')[0]
        stages[node] = pipeline_builder.add_stage(STAGE_NAMES[int(component)])
    for node, neighbors in graph.items():
        if [node] != neighbors:
            stages[node] >> [stages[neighbor] for neighbor in neighbors]
    fragment = pipeline_builder.build('test_sample_pipeline_fragment')
    sch.publish_pipeline(fragment)

    try:
        issues = json.loads(fragment._data['pipelineDefinition'])['issues']
        if issues['issueCount'] == 1:
            assert issues['pipelineIssues'][0]['message'] == 'VALIDATION_0001 - The pipeline is empty'
        else:
            assert json.loads(fragment._data['pipelineDefinition'])['issues']['issueCount'] == 0
        assert json.loads(fragment._data['pipelineDefinition'])['metadata']['labels'][0] == label
    finally:
        sch.delete_pipeline(fragment)


@pytest.mark.parametrize(
    'permutation',
    (
        ['0', 'Trash'],
        ['Dev Data Generator', '2'],
        ['Dev Data Generator', '1', 'Trash'],
        ['0_1', 'Trash'],
        ['Dev Data Generator', '1_2'],
        ['Dev Data Generator', '1_3', 'Trash'],
        ['0', 'Expression Evaluator', '2'],
        ['0', '1', 'Trash'],
        ['Dev Data Generator', '1', '2'],
        ['0', '1_3', 'Trash'],
        ['0', '1', '2'],
        ['0_1', '2'],
        ['0', '1_3', '2'],
    ),
)
def test_import_fragments_linear_pipelines(sch, sch_authoring_sdc_id, permuted_fragments, permutation):
    """Each parameter represents a linear pipeline. The numbers represent fragment names.
    e.g. ['0', '1_3', 'Trash'] represents a pipeline:
            Fragment(name='fragment_from_sdk_0') >> Fragment(name='fragment_from_sdk_1_3') >> 'Trash'.
    """
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    stages = []
    for component in permutation:
        if component in STAGE_NAMES:
            stages.append(pipeline_builder.add_stage(component))
        else:
            fragment_name = 'fragment_from_sdk_{}'
            stages.append(pipeline_builder.add_fragment(permuted_fragments.get(name=fragment_name.format(component))))
    for i in range(len(stages) - 1):
        stages[i] >> stages[i + 1]
    pipeline_name = 'test_linear_data_collector_pipeline_using_fragments_{}'.format(get_random_string())
    pipeline = pipeline_builder.build(pipeline_name)

    try:
        sch.publish_pipeline(pipeline)
        assert json.loads(pipeline._data['pipelineDefinition'])['issues']['issueCount'] == 0
    finally:
        sch.delete_pipeline(pipeline)


@pytest.mark.parametrize(
    'permutation',
    (
        {'0': ['Trash+0', 'Trash+1'], 'Trash+0': ['Trash+0'], 'Trash+1': ['Trash+1']},
        {'0': ['1', '3'], '1': ['Trash+0'], '3': ['Trash+1'], 'Trash+0': ['Trash+0'], 'Trash+1': ['Trash+1']},
        {
            '0': ['Expression Evaluator', 'Field Renamer'],
            'Expression Evaluator': ['Trash+0'],
            'Field Renamer': ['Trash+1'],
            'Trash+0': ['Trash+0'],
            'Trash+1': ['Trash+1'],
        },
        {'0': ['1', '3'], '1': ['Trash'], '3': ['Trash'], 'Trash': ['Trash']},
        {
            '0': ['Expression Evaluator', 'Field Renamer'],
            'Expression Evaluator': ['Trash'],
            'Field Renamer': ['Trash'],
            'Trash': ['Trash'],
        },
        {'0': ['1', '3'], '1': ['2'], '3': ['2'], '2': ['2']},
        {'0': ['1_3'], '1_3': ['Trash+0', 'Trash+1'], 'Trash+0': ['Trash+0'], 'Trash+1': ['Trash+1']},
        {'0': ['1_3+0', '1_3+1'], '1_3+0': ['2'], '1_3+1': ['2'], '2': ['2']},
    ),
)
def test_import_fragments_non_linear_pipelines(sch, sch_authoring_sdc_id, permuted_fragments, permutation):
    """Each parmter represents a non-linear pipeline graph. The numbers represent fragment names.
    e.g. {'0': ['Expression Evaluator', 'Field Renamer'],
          'Expression Evaluator': ['Trash+0'],
          'Field Renamer': ['Trash+1'],
          'Trash+0': ['Trash+0'],
          'Trash+1': ['Trash+1']} represents a pipeline:
                Fragment(name='fragment_from_sdk_0') >> ['Expression Evaluator', 'Field Renamer']
                'Expression Evaluator' >> 'Trash 0'
                'Field Renamer' >> 'Trash 1'
    """
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    stages = {}
    for node in permutation:
        component = node
        if '+' in node:
            component = node.split('+')[0]
        if component in STAGE_NAMES:
            stages[node] = pipeline_builder.add_stage(component)
        else:
            fragment_name = 'fragment_from_sdk_{}'
            stages[node] = pipeline_builder.add_fragment(permuted_fragments.get(name=fragment_name.format(component)))
    for node, neighbors in permutation.items():
        if [node] != neighbors:
            stages[node] >> [stages[neighbor] for neighbor in neighbors]
    pipeline = pipeline_builder.build('test_non_linear_pipeline_using_fragments')
    sch.publish_pipeline(pipeline)
    try:
        assert json.loads(pipeline._data['pipelineDefinition'])['issues']['issueCount'] == 0
    finally:
        sch.delete_pipeline(pipeline)


def test_pipeline_fragment_with_multiple_output_lanes(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(
        engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
    )
    dev_data_generator = pipeline_builder.add_stage('Dev Data Generator')
    expression_evaluator = pipeline_builder.add_stage('Expression Evaluator')
    field_renamer = pipeline_builder.add_stage('Field Renamer')
    dev_data_generator >> [expression_evaluator, field_renamer]
    fragment = pipeline_builder.build('fragment_with_multiple_output_lanes')
    sch.publish_pipeline(fragment)

    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    # fragment = sch.pipelines.get(fragment=True, name='Test Fragment')
    fragment_stage = pipeline_builder.add_fragment(fragment)

    trash1 = pipeline_builder.add_stage('Trash')
    trash2 = pipeline_builder.add_stage('Trash')

    fragment_stage >> trash1
    fragment_stage >> trash2

    pipeline = pipeline_builder.build('test_pipeline_to_test_fragment_with_multiple_output_lanes')
    sch.publish_pipeline(pipeline)

    try:
        assert json.loads(pipeline._data['pipelineDefinition'])['issues']['issueCount'] == 0
    finally:
        sch.delete_pipeline(pipeline)
        sch.delete_pipeline(fragment)


def test_pipeline_fragment_parameter_name_prefix_set_in_ui_info(sch, sch_authoring_sdc_id):
    """parameterNamePrefix is set in the uiInfo section of the stage definition."""
    PARAMETER_NAME_PREFIX = 'SOMESILLYTHING'

    fragment_builder = sch.get_pipeline_builder(
        engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
    )
    fragment_builder.add_stage('Expression Evaluator')
    fragment = fragment_builder.build('Fragment for parameter name prefix test')
    try:
        sch.publish_pipeline(fragment)

        pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
        fragment_stage = pipeline_builder.add_fragment(fragment, parameter_name_prefix=PARAMETER_NAME_PREFIX)
        assert fragment_stage._data['uiInfo']['parameterNamePrefix'] == PARAMETER_NAME_PREFIX
    finally:
        sch.delete_pipeline(fragment)


def test_update_pipelines_with_different_fragment_version(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(
        engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
    )
    pipeline_builder.add_stage('Dev Data Generator')
    fragment = pipeline_builder.build('fragment to update')
    sch.publish_pipeline(fragment)

    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    dev = pipeline_builder.add_fragment(fragment)
    trash = pipeline_builder.add_stage('Trash')
    dev >> trash
    pipeline = pipeline_builder.build('pipeline using old version of fragment')
    sch.publish_pipeline(pipeline)

    fragment.name = 'updated fragment'
    sch.publish_pipeline(fragment)

    try:
        from_fragment_version = fragment.commits.get(version='1')
        to_fragment_version = fragment.commits.get(version='2')
        assert len(pipeline.commits) == 1
        sch.update_pipelines_with_different_fragment_version(
            pipelines=[pipeline], from_fragment_version=from_fragment_version, to_fragment_version=to_fragment_version
        )
        assert len(pipeline.commits) == 2
    finally:
        sch.delete_pipeline(sch.pipelines.get(pipeline_id=pipeline.pipeline_id))
        sch.delete_pipeline(sch.pipelines.get(pipeline_id=fragment.pipeline_id, fragment=True))


def test_add_fragment_with_event_lane(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(
        engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
    )
    dev = pipeline_builder.add_stage('Dev Data Generator', type='origin')
    trash = pipeline_builder.add_stage('Trash')

    dev >= trash
    fragment = pipeline_builder.build('fragment to update')
    sch.publish_pipeline(fragment)

    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)

    fragment_stage = pipeline_builder.add_fragment(fragment)
    trash = pipeline_builder.add_stage('Trash')

    fragment_stage >> trash
    pipeline = pipeline_builder.build('SDK pipeline using fragment with event lane {}'.format(get_random_string()))

    try:
        sch.publish_pipeline(pipeline)
        pipeline = sch.pipelines.get(name=pipeline.name)

        # assert that the eventlane is {instance_name}_EventLane in ControlHub and that the add_fragment
        # method sets this name accurately. If this is not set properly the event lane will be stripped in ControlHub
        for stage in pipeline._pipeline_definition['fragments'][0]['stages']:
            if stage['uiInfo']['label'] == 'Dev Data Generator 1' and stage['eventLanes']:
                assert '{}_EventLane'.format(stage['instanceName']) == stage['eventLanes'][0]
    finally:
        sch.delete_pipeline(sch.pipelines.get(pipeline_id=pipeline.pipeline_id))
        sch.delete_pipeline(sch.pipelines.get(pipeline_id=fragment.pipeline_id))


@pytest.fixture(scope='module')
def permuted_fragments(sch, sch_authoring_sdc_id):
    # Indices of STAGE_NAMES
    permutations = [[0], [1], [2], [3], [0, 1], [1, 2], [1, 3]]
    fragments = SeekableList()
    for perm in permutations:
        pipeline_builder = sch.get_pipeline_builder(
            engine_type='data_collector', engine_id=sch_authoring_sdc_id, fragment=True
        )
        stages = []
        # Add all stages to pipeline builder
        for idx in perm:
            stages.append(pipeline_builder.add_stage(STAGE_NAMES[idx]))
        # Connect them in a linear way
        for i in range(len(stages) - 1):
            stages[i] >> stages[i + 1]
        fragment = pipeline_builder.build('fragment_from_sdk_{}'.format('_'.join(map(str, perm))))
        sch.publish_pipeline(fragment)
        fragments.append(fragment)

    try:
        yield fragments
    finally:
        for fragment in fragments:
            sch.delete_pipeline(fragment)



# Copyright 2022 StreamSets Inc.
# fmt: off
import pytest

from streamsets.sdk.utils import get_random_string

# fmt: on


@pytest.fixture(scope="module")
def simple_pipeline(sch, sch_authoring_sdc_id):
    """A trivial pipeline:

    dev_data_generator >> trash
    """
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)

    dev_data_generator = pipeline_builder.add_stage('Dev Data Generator')
    trash = pipeline_builder.add_stage('Trash')
    dev_data_generator >> trash
    pipeline = pipeline_builder.build('simple_draft_{}'.format(get_random_string()))
    sch.publish_pipeline(pipeline, draft=True)

    try:
        yield pipeline
    finally:
        sch.api_client.delete_pipeline(pipeline.pipeline_id)


@pytest.fixture(scope="module")
def simple_draft_run(sch, sch_authoring_sdc_id, simple_pipeline):
    """A trivial draft run:"""
    start_draft_command = sch.start_draft_run(simple_pipeline)
    start_draft_command.wait_for_job_status('ACTIVE')
    draft_run_obj = sch.draft_runs.get(search="id=='{}'".format(start_draft_command.response.json()['jobId']))

    try:
        yield draft_run_obj
    finally:
        sch.stop_draft_run(draft_run_obj)
        sch.delete_draft_run(draft_run_obj)


def test_draft_run_snapshot(sch, simple_draft_run):
    simple_draft_run.capture_snapshot()
    simple_draft_run.wait_for_finished_snapshot()
    snapshots = simple_draft_run.snapshots
    assert snapshots[0].name == "Snapshot1"
    assert snapshots[-1].batches is not None

    simple_draft_run.capture_snapshot()
    simple_draft_run.wait_for_finished_snapshot()
    snapshots = simple_draft_run.snapshots
    assert len(snapshots) == 2
    assert snapshots[-1].id is not None
    assert snapshots[1].time_stamp > snapshots[0].time_stamp

    simple_draft_run.remove_snapshot(snapshots[0])
    snapshots = simple_draft_run.snapshots
    assert len(snapshots) == 1

    simple_draft_run.remove_snapshot(snapshots[0])
    assert len(simple_draft_run.snapshots) == 0


def test_draft_run_logs(sch, simple_draft_run):
    logs = simple_draft_run.get_logs()
    assert logs is not None



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2024

# fmt: off
import pytest
import requests

from streamsets.sdk.sch_models import Deployment
from streamsets.sdk.utils import get_random_string

# fmt: on


def test_add_deployment_adds_disabled_engine(args, test_environment, sch):
    # Check if there's disabled engines that can be used to create a deployment
    configuration = sch.api_client.get_organization_configuration(sch.organization).response.json()
    for config in configuration:
        if config["id"] == "dpm.disabled.engine.version.override.list":
            if not config['possibleValues']:
                pytest.skip("Currently unable to move disabled engines into the override list")
            engine_version_id = config['possibleValues'][0]

    engine_version_id_split = engine_version_id.split(":")
    engine_type = engine_version_id_split[0]
    engine_version = engine_version_id_split[1]
    engine_build = engine_version_id_split[-1]

    # Verify that the engine used to build the deployment is disabled
    assert sch.engine_versions.get(id="{}:{}::{}".format(engine_type, engine_version, engine_build)).disabled

    builder = sch.get_deployment_builder(deployment_type='SELF')
    deployment = builder.build(
        deployment_name='SDK Test Deployment SDC_Docker {}'.format(get_random_string()),
        engine_type=engine_type,
        engine_version=engine_version,
        engine_build=engine_build,
        environment=test_environment,
    )
    sch.add_deployment(deployment)
    try:
        retrieved_deployment = sch.deployments.get(deployment_name=deployment.deployment_name)
        assert isinstance(retrieved_deployment, Deployment)
    finally:
        sch.delete_deployment(deployment)


def test_delete_active_deployment(test_environment, sch):
    EXPECTED_ERROR_MESSAGE = "Cannot delete deployment that is currently ACTIVE or ACTIVATION_ERROR - stop it first"
    builder = sch.get_deployment_builder(deployment_type='SELF')
    deployment = builder.build(
        deployment_name='SDK Test Deployment SDC_Docker {}'.format(get_random_string()),
        engine_type="DC",
        engine_version="5.1.0",
        environment=test_environment,
    )
    sch.add_deployment(deployment)
    sch.start_deployment(deployment)
    assert deployment.state == 'ACTIVE'
    with pytest.raises(requests.exceptions.HTTPError) as error_message:
        sch.delete_deployment(deployment, stop=False)
        assert EXPECTED_ERROR_MESSAGE == error_message["ISSUES"][0]["message"]
    sch.delete_deployment(deployment)
    with pytest.raises(ValueError):
        sch.deployments.get(deployment_id=deployment.deployment_id)



from streamsets.sdk.utils import get_random_string

GET_ENGINE_RESPONSE_COLLECTOR_JSON = {
    'id': get_random_string(),
    'organization': '1a1b7cf4-862e-11ed-94d5-41a005fd7967',
    'httpUrl': 'http://f8728e856446:18630',
    'version': '5.6.0',
    'labels': [],
    'reportedLabels': ['deployment'],
    'pipelinesCommitted': [],
    'lastReportedTime': 1697835654564,
    'startUpTime': 1696872736374,
    'offsetProtocolVersion': 2,
    'edge': False,
    'cpuLoad': 0.7079236552920763,
    'totalMemory': 16606822400.0,
    'usedMemory': 581667472.0,
    'memoryUsedPercentage': 0.03502581396908297,
    'pipelinesCount': 0,
    'responding': True,
    'idSeparator': '__',
    'executorType': 'COLLECTOR',
    'maxCpuLoad': 80.0,
    'maxMemoryUsed': 100.0,
    'maxPipelinesRunning': 1000000,
    'cspDeploymentId': '14dc3111-b89d-488a-b9c1-4c6d7a767c39:1a1b7cf4-862e-11ed-94d5-41a005fd7967',
    'deploymentId': None,
    'buildTime': 1687277400000,
    'overrideEngineMaxes': False,
    'lastShutdownTimestamp': None,
}

GET_ENGINE_RESPONSE_TRANSFORMER_JSON = {
    'id': get_random_string(),
    'organization': '1a1b7cf4-862e-11ed-94d5-41a005fd7967',
    'httpUrl': 'http://f8728e856446:18630',
    'version': '5.6.0',
    'labels': [],
    'reportedLabels': ['deployment'],
    'pipelinesCommitted': [],
    'lastReportedTime': 1697835654564,
    'startUpTime': 1696872736374,
    'offsetProtocolVersion': 2,
    'edge': False,
    'cpuLoad': 0.7079236552920763,
    'totalMemory': 16606822400.0,
    'usedMemory': 581667472.0,
    'memoryUsedPercentage': 0.03502581396908297,
    'pipelinesCount': 0,
    'responding': True,
    'idSeparator': '__',
    'executorType': 'TRANSFORMER',
    'maxCpuLoad': 80.0,
    'maxMemoryUsed': 100.0,
    'maxPipelinesRunning': 1000000,
    'cspDeploymentId': '14dc3111-b89d-488a-b9c1-4c6d7a767c39:1a1b7cf4-862e-11ed-94d5-41a005fd7967',
    'deploymentId': None,
    'buildTime': 1687277400000,
    'overrideEngineMaxes': False,
    'lastShutdownTimestamp': None,
}



import json

# Referenced Data for function mock_connection(dummy_sch)
mock_connection_library_definition = {
    'verifierDefinitions': [
        {
            'verifierClass': 'com.streamsets.pipeline.lib.jdbc.connection.JdbcConnectionVerifier',
            'verifierConnectionFieldName': 'connection',
            'verifierConnectionSelectionFieldName': 'connectionSelection',
            'verifierType': 'STREAMSETS_JDBC',
            'library': 'streamsets-datacollector-sdc-snowflake-lib',
        },
        {
            'verifierClass': 'com.streamsets.pipeline.lib.jdbc.connection.JdbcConnectionVerifier',
            'verifierConnectionFieldName': 'connection',
            'verifierConnectionSelectionFieldName': 'connectionSelection',
            'verifierType': 'STREAMSETS_JDBC',
            'library': 'streamsets-datacollector-jdbc-lib',
        },
    ]
}

CONNECTION_INTERNAL_JSON = {
    'id': 'random-connection-id',
    'sdcId': 'random-sdc-id',
    'libraryDefinition': json.dumps(mock_connection_library_definition),
}



# Copyright 2022 StreamSets Inc.

# fmt: off
import traceback

import pytest

from streamsets.sdk.sch_models import KubernetesAgentEvent, KubernetesAgentEvents
from streamsets.sdk.utils import get_random_string

# fmt: on


@pytest.fixture(scope="module")
def sample_environment(sch):
    """A sample Kubernetes environment."""
    try:
        environment_builder = sch.get_environment_builder(environment_type='KUBERNETES')
        environment = environment_builder.build(
            environment_name='Sample K8s Environment {}'.format(get_random_string()),
            environment_type='KUBERNETES',
            environment_tags=['kubernetes-tag-{}'.format(get_random_string())],
            allow_nightly_engine_builds=True,
        )
        environment.kubernetes_namespace = 'streamsets test'
        sch.add_environment(environment)
        # The Kubernetes environment was successfully added to the SCH
        success = True

        sch.activate_environment(environment)

    except Exception as ex:
        success = False
        error_msg = (
            "Unexpected error while trying to create a 'sample_environment' Kubernetes environment : " "{}\n\n{}"
        ).format(ex, traceback.format_exc())

    if success:
        try:
            yield environment
        finally:
            sch.delete_environment(environment)
    else:
        raise AssertionError(error_msg)


def test__given_sample_environment_and_valid_environment_id__when_get_kubernetes_environment_agent_events_then_ok(
    sch, sample_environment
):
    # Given
    environment_id = sample_environment.environment_id

    # When
    # NOTICE that this test explicitly verifies that the SCH's API class implements the whole communication with the
    # Kubernetes environment to retrieve its events.
    events = sch.api_client.get_kubernetes_environment_agent_events(environment_id)

    # Then
    # Check that the response includes the environment events in a JSON body
    event_body = events.response.json()

    assert len(event_body["data"]) == 2
    assert event_body["data"][0]["details"] == "Activated agent token"
    assert event_body["data"][1]["details"] == "Waiting for Agent to connect..."


def test__given_sample_environment_and_kubernetes_agent_events_and_no_kwargs__when_get_all_results_from_api__then_ok(
    sch, sample_environment
):
    # Given
    kwargs = {}
    kubernetes_agent_events = KubernetesAgentEvents(sch, sample_environment.environment_id)

    # When
    # NOTICE that this test explicitly verifies that the events' collection (i.e., KubernetesAgentEvents) implements the
    # whole communication with the Kubernetes environment to retrieve its events.
    result = kubernetes_agent_events._get_all_results_from_api(**kwargs)

    # Then
    # Check that the response follows the expected format.
    assert len(result) == 4  # i.e., kwargs, class_kwargs, class_type, and results
    assert result.kwargs == {}
    assert result.class_kwargs == {}
    assert KubernetesAgentEvent.__name__ in str(result.class_type)

    # Check that the response from the API contains the two events any fresh-new Kubernetes Environment has got.
    # Notice that the API is paginated, and we have received a single page containing the two events.
    assert result.results["len"] == 2
    assert result.results["offset"] == 0
    assert result.results["totalCount"] == result.results["len"]
    assert len(result.results["data"]) == result.results["len"]
    assert result.results["data"][0]["details"] == "Activated agent token"
    assert result.results["data"][1]["details"] == "Waiting for Agent to connect..."


def test__given_sample_environment_and_no_kwargs__when_get_all_results_from_api__then_ok(sample_environment):
    # Given
    kwargs = {}

    # When
    # NOTICE that this test explicitly verifies that the events are an attribute of the environment
    result = sample_environment.agent_event_logs._get_all_results_from_api(**kwargs)

    # Then
    # Check that the response follows the expected format.
    assert len(result) == 4  # i.e., kwargs, class_kwargs, class_type, and results
    assert result.kwargs == {}
    assert result.class_kwargs == {}
    assert KubernetesAgentEvent.__name__ in str(result.class_type)

    # Check that the response from the API contains the two events any fresh-new Kubernetes Environment has got.
    # Notice that the API is paginated, and we have received a single page containing the two events.
    assert result.results["len"] == 2
    assert result.results["offset"] == 0
    assert result.results["totalCount"] == result.results["len"]
    assert len(result.results["data"]) == result.results["len"]
    assert result.results["data"][0]["details"] == "Activated agent token"
    assert result.results["data"][1]["details"] == "Waiting for Agent to connect..."



# Copyright Streamsets 2023

import copy
import logging

import pytest
from requests.exceptions import HTTPError

logger = logging.getLogger(__name__)


# Session scope because all tests run on the same organization
@pytest.fixture(scope="session")
def org_config(sch):
    org_config = sch.organizations.get(id=sch.organization).configuration

    original_config = copy.deepcopy(org_config)
    try:
        yield org_config
    finally:
        # reset org configs to the original config
        for key, value in original_config.items():
            # check to avoid making extra api calls
            if org_config[key] != value:
                try:
                    org_config[key] = value
                except HTTPError as e:
                    # certain keys for organizations cannot be updated on platform by the organization itself.
                    # we just catch and log these, in case something weird happened, and we want to debug.
                    logger.error("Failed to update key {} with error {}".format(key, e))


def test_update_org_config_fails_for_disabled_setting(org_config):
    KEY_THAT_CANNOT_BE_MODIFIED_BY_ORG_ADMIN = 'Maximum number of topology commits for the organization'
    with pytest.raises(HTTPError):
        org_config[KEY_THAT_CANNOT_BE_MODIFIED_BY_ORG_ADMIN] = 14



# Copyright 2019 StreamSets Inc.

# fmt: off
import pytest

from streamsets.sdk.exceptions import InvalidVersionError
from streamsets.sdk.utils import Version

# fmt: on


# Version Parameter: version string, tuple of expected parsed values
@pytest.mark.parametrize(
    'raw_string, expected',
    (
        # bare version numbers
        ['3.0.0.0', (None, [3, 0, 0, 0], None)],
        ['3.0.1', (None, [3, 0, 1, 0], None)],
        ['12.0', (None, [12, 0, 0, 0], None)],
        ['0.0.0', (None, [0, 0, 0, 0], None)],
        ['1.12.24', (None, [1, 12, 24, 0], None)],
        ['0.0.0.1', (None, [0, 0, 0, 1], None)],
        # from int/float
        [3.8, (None, [3, 8, 0, 0], None)],
        [3, (None, [3, 0, 0, 0], None)],
        # with version specifier
        ['3.0.1-SNAPSHOT', (None, [3, 0, 1, 0], 'SNAPSHOT')],
        ['3.8.2-RC2', (None, [3, 8, 2, 0], 'RC2')],
        ['3.8.2RC2', (None, [3, 8, 2, 0], 'RC2')],
        ['3.8.2SNAPSHOT', (None, [3, 8, 2, 0], 'SNAPSHOT')],
        ['3.8.2.42SNAPSHOT', (None, [3, 8, 2, 42], 'SNAPSHOT')],
        # with a name
        ['foo-3.0.1', ("foo", [3, 0, 1, 0], None)],
        ['foo3.0.1RC2', ("foo", [3, 0, 1, 0], "RC2")],
    ),
)
def test_version_parser(raw_string, expected):
    parsed_version = Version(raw_string)
    assert (parsed_version.name, parsed_version.version, parsed_version.specifier) == expected


@pytest.mark.parametrize(
    "initial, expected",
    (
        ["3.0", "<Version: None, 3.0.0.0, None>"],
        ["3.0.1RC2", "<Version: None, 3.0.1.0, 'RC2'>"],
        ["foo3.0.1RC2", "<Version: 'foo', 3.0.1.0, 'RC2'>"],
        ["foo-3.0.1-RC2", "<Version: 'foo', 3.0.1.0, 'RC2'>"],
        ["foo-3.0.1", "<Version: 'foo', 3.0.1.0, None>"],
        ["foo-3.0.1.4", "<Version: 'foo', 3.0.1.4, None>"],
    ),
)
def test_version_repr(initial, expected):
    assert repr(Version(initial)) == expected


@pytest.mark.parametrize(
    "initial, expected",
    (
        ["NotAVersion", InvalidVersionError],
        ["3.2.1.", InvalidVersionError],
        [None, InvalidVersionError],
        [[], InvalidVersionError],
    ),
)
def test_invalid_version(initial, expected):
    with pytest.raises(expected):
        Version(initial)


@pytest.mark.parametrize(
    "initial, expected",
    (
        ["3.0", "3.0.0.0"],
        ["3.0.1RC2", "3.0.1.0-RC2"],
        ["foo3.0.1RC2", "foo-3.0.1.0-RC2"],
        ["foo-3.0.1", "foo-3.0.1.0"],
        ["foo-3.0.1.4", "foo-3.0.1.4"],
    ),
)
def test_version_str(initial, expected):
    assert str(Version(initial)) == expected


@pytest.mark.parametrize(
    'lhs, rhs, expected',
    (
        # strict version ordering checks
        [Version('3.0.0.0'), Version('3.0'), True],
        [Version('3.0.1'), Version('3.0.1.0'), True],
        [Version('3.0.1.0'), Version('3.0.1.4'), False],
        [Version('3.1.0'), Version('3.0.1'), False],
        # ignore any release specifiers
        [Version('3.8.0-latest'), Version('3.8-SNAPSHOT'), True],
        [Version('3.8.0-latest'), Version('3.8'), True],
        # version-to-object comparisons
        [Version("3.1"), "3.1", True],
        [Version("3.1"), "3.2", False],
        [Version(5), 5, True],
        [Version(5), 6, False],
        [Version("3.1-RC4"), "3.1-snapshot2", True],
    ),
)
def test_version_object_equality(lhs, rhs, expected):
    assert (lhs == rhs) == expected


@pytest.mark.parametrize(
    'lhs, rhs, expected',
    (
        # strict version ordering checks
        [Version('3.0.0.0'), Version('3.0.0.1'), True],
        [Version('3.0.1'), Version('3.0.2.0'), True],
        [Version('3.0.0'), Version('12.0.1.0'), True],
        [Version('3.5.1'), Version('3.5.0'), False],
        # ignore any release specifiers
        [Version('3.8-latest'), Version('3.8.1-SNAPSHOT'), True],
        [Version('3.8-post3'), Version('3.8.1-alpha12'), True],
        # version-to-object comparisons
        [Version("3.1"), "3.1.4", True],
        [Version(5), 6, True],
        [Version("3.1-RC2"), "3.2-snapshot2", True],
        [Version("3.1-RC2"), "3.1-snapshot2", False],
    ),
)
def test_version_object_comparison(lhs, rhs, expected):
    assert (lhs < rhs) == expected



TEST_VALIDATE_PIPELINE_STAGES_PASSES_FOR_VALID_PIPELINE_WITH_VALID_STAGES_JSON = {
    'instanceName': 'DevDataGenerator_1',
    'library': 'streamsets-datacollector-dev-lib',
    'stageName': 'com_streamsets_pipeline_stage_devtest_RandomDataGeneratorSource',
    'stageVersion': '6',
    'configuration': [
        {'name': 'delay', 'value': 1000},
        {'name': 'numberOfRecords', 'value': 0},
        {'name': 'eventName', 'value': 'generated-event'},
        {'name': 'batchSize', 'value': 1000},
        {'name': 'headerAttributes', 'value': []},
        {
            'name': 'dataGenConfigs',
            'value': [{'type': 'STRING', 'precision': 10, 'scale': 2, 'fieldAttributes': []}],
        },
        {'name': 'numThreads', 'value': 1},
        {'name': 'rootFieldType', 'value': 'MAP'},
        {'name': 'stageOnRecordError', 'value': 'TO_ERROR'},
    ],
    'uiInfo': {
        'colorIcon': 'Origin_Dev_Data_Generator.png',
        'outputStreamLabels': None,
        'yPos': 50,
        'stageType': 'SOURCE',
        'rawSource': None,
        'icon': 'dev.png',
        'description': '',
        'inputStreamLabels': None,
        'label': 'Dev Data Generator 1',
        'xPos': 60,
        'displayMode': 'BASIC',
    },
    'inputLanes': [],
    'outputLanes': ['DevDataGenerator_1OutputLane17048194517380'],
    'eventLanes': [],
    'services': [],
}

TEST_VALIDATE_PIPELINE_STAGES_FAILS_FOR_INVALID_PIPELINE_WITH_INVALID_STAGES_JSON = {
    'instanceName': 'AmazonS3_01',
    'library': 'streamsets-datacollector-aws-lib',
    'stageName': 'com_streamsets_pipeline_stage_destination_s3_AmazonS3DTarget',
    'stageVersion': '15',
    'configuration': [
        {'name': 's3TargetConfigBean.s3Config.connection.awsConfig.awsSecretAccessKey', 'value': None},
        {'name': 's3TargetConfigBean.tmConfig.threadPoolSize', 'value': 10},
    ],
    'uiInfo': {
        'colorIcon': 'Destination_Amazon_S3.png',
        'description': '',
        'label': 'Amazon S3 1',
        'xPos': 280,
        'yPos': 50,
        'stageType': 'TARGET',
    },
    'inputLanes': [],
    'outputLanes': [],
    'eventLanes': [],
    'services': [
        {
            'service': 'com.streamsets.pipeline.api.service.dataformats.DataFormatGeneratorService',
            'serviceVersion': '3',
            'configuration': [
                {'name': 'displayFormats', 'value': 'AVRO,BINARY,DELIMITED,JSON,PROTOBUF,SDC_JSON,TEXT,WHOLE_FILE'},
            ],
        }
    ],
}



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2024

from copy import deepcopy

import pytest

from streamsets.sdk import ControlHub
from streamsets.sdk.sch_api import ApiClient
from streamsets.sdk.sch_models import SelfManagedDeployment

from .resources.self_managed_deployments_data import DUMMY_DEPLOYMENT_JSON

VALID_INSTALL_SCRIPT = "Install Script"


@pytest.fixture(scope="function")
def dummy_deployment_data():
    data = deepcopy(DUMMY_DEPLOYMENT_JSON)
    return data


class MockControlHub(ControlHub):
    def __init__(self, *args, **kwargs):
        self.api_client = MockApiClient()
        pass  # do not call super()


class MockResponse:
    def __init__(self, json_response):
        self.response = json_response

    def json(self):
        return self.response

    @property
    def text(self):
        return self.response


class MockCommand:
    def __init__(self, json_response):
        self._response = json_response

    @property
    def response(self):
        return MockResponse(self._response)


class MockApiClient(ApiClient):
    def __init__(self, *args, **kwargs):
        pass  # do not call super()

    def get_engine_version(self, engine_version_id):
        return MockCommand(
            {
                "id": "DC:5.10.0::RC3",
                "engineType": "DC",
                "engineVersion": "5.10.0",
                "creator": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                "createTime": 1711051750490,
                "lastModifiedBy": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                "lastModifiedOn": 1711051750490,
                "defaultJavaVersion": 8,
                "supportedJavaVersions": "8,17",
            }
        )

    def get_self_managed_deployment_install_command(
        self, deployment_id, install_mechanism='DEFAULT', install_type=None, java_version=None
    ):
        return MockCommand(VALID_INSTALL_SCRIPT)


@pytest.fixture(scope="function")
def self_managed_deployment(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    return deployment


def test_invalid_java_version_install_script(self_managed_deployment):
    with pytest.raises(Exception):
        self_managed_deployment.install_script(java_version='3')


def test_supported_java_version_install_script(self_managed_deployment):
    assert self_managed_deployment.install_script(java_version='17') == VALID_INSTALL_SCRIPT


def test_default_java_version_install_script(self_managed_deployment):
    assert self_managed_deployment.install_script() == VALID_INSTALL_SCRIPT


@pytest.mark.parametrize("install_mechanism", [mechanism for mechanism in SelfManagedDeployment.InstallMechanism])
def test_valid_install_mechanism(self_managed_deployment, install_mechanism):
    install_script = self_managed_deployment.install_script(install_mechanism=install_mechanism.value)
    assert type(install_script) is str
    assert len(install_script) > 0


@pytest.mark.parametrize("install_type", [install_type for install_type in SelfManagedDeployment.InstallType])
def test_valid_install_type(self_managed_deployment, install_type):
    install_script = self_managed_deployment.install_script(install_type=install_type.value)
    assert install_script == VALID_INSTALL_SCRIPT


def test_invalid_install_mechanism(self_managed_deployment):
    invalid_install_mechanism = "foo"

    with pytest.raises(ValueError):
        self_managed_deployment.install_script(install_mechanism=invalid_install_mechanism)


def test_invalid_install_type(self_managed_deployment):
    invalid_install_type = "foo"

    with pytest.raises(ValueError):
        self_managed_deployment.install_script(install_type=invalid_install_type)



# fmt: off
import json
import warnings
from copy import deepcopy

import pytest

from streamsets.sdk.models import Configuration
from streamsets.sdk.sch_models import Connection, Pipeline, SchSdcStage

from .resources.configuration_data import DUMMY_CONNECTION_JSON, DUMMY_DESTINATION_STAGE_JSON

# fmt: on


class DummyPipelineBuilder:
    """
    A Dummy Pipeline Class Builder
    """

    def __init__(self):
        self._config_key = 'pipelineBuilder'
        self._pipeline = {self._config_key: {'stages': []}}


@pytest.fixture(scope="function")
def dummy_connection(mocker):
    connection_json = deepcopy(DUMMY_CONNECTION_JSON)
    return Connection(connection_json, mocker.Mock())


@pytest.fixture(scope="function")
def dummy_pipeline(mocker):
    pipeline_json = {'pipelineId': 1, 'commitId': None, 'name': 'Test Pipeline', 'version': 1}

    configuration = [
        {'name': 'transformerEmrConnection.retryPolicyConfig.baseDelay', 'value': 100},
        {'name': 'emrServerlessConnection.retryPolicyConfig.maxBackoff', 'value': 20000},
        {'name': 'googleCloudConfig.workerCount', 'value': 2},
    ]
    pipeline_definition_json = {'configuration': configuration}

    rules_definition_json = {}

    config_definitions = [
        {
            'label': 'Retry Base Delay',
            'fieldName': 'baseDelay',
            'name': 'transformerEmrConnection.retryPolicyConfig.baseDelay',
        },
        {
            'label': 'Max Backoff',
            'fieldName': 'maxBackoff',
            'name': 'emrServerlessConnection.retryPolicyConfig.maxBackoff',
        },
        {'label': 'Worker Count', 'fieldName': 'workerCount', 'name': 'googleCloudConfig.workerCount'},
    ]
    library_definitions_json = json.dumps({'pipeline': [{'configDefinitions': config_definitions}]})

    builder = DummyPipelineBuilder()

    pipeline = Pipeline(
        pipeline=pipeline_json,
        builder=builder,
        pipeline_definition=pipeline_definition_json,
        rules_definition=rules_definition_json,
        library_definitions=library_definitions_json,
        control_hub=mocker.Mock(),
    )
    pipeline.fragment = False

    return pipeline


@pytest.fixture(scope='function')
def dummy_destination_stage():
    stage_json = deepcopy(DUMMY_DESTINATION_STAGE_JSON)
    stage = SchSdcStage(stage=stage_json)
    return stage


def test_configuration_sanity(dummy_pipeline):
    assert isinstance(dummy_pipeline.configuration, Configuration)


def test_get_configuration_value(dummy_pipeline):
    # test against attribute.python_name, attribute['python_name'] & attribute['camelcase_name']
    assert dummy_pipeline.configuration.retry_base_delay == 100
    assert dummy_pipeline.configuration['retry_base_delay'] == 100
    assert dummy_pipeline.configuration['transformerEmrConnection.retryPolicyConfig.baseDelay'] == 100

    # make sure the first element in underlying data is what is being tested
    assert (
        dummy_pipeline._pipeline_definition['configuration'][0]['name']
        == 'transformerEmrConnection.retryPolicyConfig.baseDelay'
    )
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 100


def test_set_configuration_value_setattr_pythonic_name(dummy_pipeline):
    # make sure the first element in underlying data is transformerEmrConnection.retryPolicyConfig.baseDelay
    assert (
        dummy_pipeline._pipeline_definition['configuration'][0]['name']
        == 'transformerEmrConnection.retryPolicyConfig.baseDelay'
    )

    # test base case where the value is 100
    assert dummy_pipeline.configuration.retry_base_delay == 100
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 100

    # test against attribute.python_name and that the change is reflected in the underlying data
    dummy_pipeline.configuration.retry_base_delay = 1
    assert dummy_pipeline.configuration.retry_base_delay == 1
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 1


def test_set_configuration_value_setitem_pythonic_name(dummy_pipeline):
    # make sure the first element in underlying data is transformerEmrConnection.retryPolicyConfig.baseDelay
    assert (
        dummy_pipeline._pipeline_definition['configuration'][0]['name']
        == 'transformerEmrConnection.retryPolicyConfig.baseDelay'
    )

    # test base case where the value is 100
    assert dummy_pipeline.configuration.retry_base_delay == 100
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 100

    # test against attribute['python_name'] and that the change is reflected in the underlying data
    dummy_pipeline.configuration['retry_base_delay'] = 5
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 5


def test_set_configuration_value_setattr_camelcase_name(dummy_pipeline):
    # make sure the first element in underlying data is transformerEmrConnection.retryPolicyConfig.baseDelay
    assert (
        dummy_pipeline._pipeline_definition['configuration'][0]['name']
        == 'transformerEmrConnection.retryPolicyConfig.baseDelay'
    )

    # test base case where the value is 100
    assert dummy_pipeline.configuration.retry_base_delay == 100
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 100

    # test against attribute['camelcase_name'] and that the change is reflected in the underlying data
    dummy_pipeline.configuration['transformerEmrConnection.retryPolicyConfig.baseDelay'] = 400
    assert dummy_pipeline._pipeline_definition['configuration'][0]['value'] == 400


def test_configuration_contains(dummy_pipeline):
    # Make sure data returned from the configuration attribute is same as the underlying data
    assert dummy_pipeline.configuration.__contains__('retry_base_delay') is True
    # test the control_hub backend name as well
    assert dummy_pipeline.configuration.__contains__('transformerEmrConnection.retryPolicyConfig.baseDelay') is True
    assert dummy_pipeline.configuration.__contains__('random_value_that_does_not_exist') is False


def test_configuration_items(dummy_pipeline):
    excepted_items = {'retry_base_delay': 100, 'max_backoff': 20000, 'worker_count': 2}

    items = dummy_pipeline.configuration.items()

    assert items is not None
    assert len(items) == len(excepted_items)

    for name, value in items:
        assert name in excepted_items.keys()
        assert value == excepted_items[name]


def test_configuration_get(dummy_pipeline):
    item_python = dummy_pipeline.configuration.get('retry_base_delay')
    assert item_python == 100

    item_backend_name = dummy_pipeline.configuration.get('transformerEmrConnection.retryPolicyConfig.baseDelay')
    assert item_backend_name == 100


def test_configuration_update_pythonic_name(dummy_pipeline):
    # test with pythonic name
    dummy_pipeline.configuration.update({'retry_base_delay': 55, 'max_backoff': 90})
    assert dummy_pipeline.configuration.retry_base_delay == 55
    assert dummy_pipeline.configuration.max_backoff == 90


def test_configuration_update_camelcase_name(dummy_pipeline):
    # test with camelcase backend name
    dummy_pipeline.configuration.update(
        {
            'transformerEmrConnection.retryPolicyConfig.baseDelay': 1,
            'emrServerlessConnection.retryPolicyConfig.maxBackoff': 5,
        }
    )
    assert dummy_pipeline.configuration.retry_base_delay == 1
    assert dummy_pipeline.configuration.max_backoff == 5


@pytest.mark.parametrize(
    "test_config, expected_value",
    [
        ({'name': 'dummy_value', 'value': 'false', 'type': 'boolean'}, False),
        ({'name': 'dummy_value', 'value': '123', 'type': 'integer'}, 123),
        ({'name': 'dummy_value', 'value': 'random value as string', 'type': 'dummy_type'}, "random value as string"),
    ],
)
def test_configuration_convert_value(dummy_pipeline, test_config, expected_value):
    value = dummy_pipeline.configuration._convert_value(test_config)
    assert value == expected_value


def test_service_configuration_under_configuration_attribute(dummy_destination_stage):
    # assert that data dataFormat exists in underlying stage data
    assert dummy_destination_stage._data['services'][0]['configuration'][0]['name'] == 'dataFormat'
    assert dummy_destination_stage._data['services'][0]['configuration'][0]['value'] == 'BINARY'

    # assert dataFormat shows up in service configuration
    assert 'dataFormat' in (
        dummy_destination_stage.services[
            'com.streamsets.pipeline.api.service.' 'dataformats.DataFormatGeneratorService'
        ]
    )

    # assert that service config shows up in stage configuration getter
    assert dummy_destination_stage.configuration['dataFormat'] == 'BINARY'

    dummy_destination_stage.configuration['dataFormat'] = 'JSON'

    # assert value has been changed
    assert (
        dummy_destination_stage.services[
            'com.streamsets.pipeline.api.service.' 'dataformats.DataFormatGeneratorService'
        ]['dataFormat']
        == 'JSON'
    )
    assert dummy_destination_stage._data['services'][0]['configuration'][0]['value'] == 'JSON'
    assert dummy_destination_stage.configuration['dataFormat'] == 'JSON'


def test_connection_configuration_sanity(dummy_connection):
    assert dummy_connection._connection_definition_internal.configuration


def test_connection_configuration_python_name_as_attribute(dummy_connection):
    configuration = dummy_connection._connection_definition_internal.configuration
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is False
    assert configuration.use_snowflake_role is False

    dummy_connection._connection_definition_internal.configuration.use_snowflake_role = True

    # Validate
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is True
    assert configuration.use_snowflake_role is True


def test_connection_configuration_python_name_as_key(dummy_connection):
    configuration = dummy_connection._connection_definition_internal.configuration
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is False
    assert configuration['use_snowflake_role'] is False

    dummy_connection._connection_definition_internal.configuration['use_snowflake_role'] = True

    # Validate
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is True
    assert configuration['use_snowflake_role'] is True


def test_connection_configuration_camel_case_name(dummy_connection):
    configuration = dummy_connection._connection_definition_internal.configuration
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is False
    assert configuration['useSnowflakeRole'] is False

    configuration['useSnowflakeRole'] = True

    # Validate
    assert configuration._data[0][2]['name'] == 'useSnowflakeRole'
    assert configuration._data[0][2]['value'] is True
    assert configuration['useSnowflakeRole'] is True


@pytest.mark.parametrize(
    "value, flag",
    [
        ([{}], False),
        ([[{}], [{}]], True),
    ],
)
def test_configuration_list_with_empty_dictionary(value, flag):
    with pytest.raises(TypeError):
        Configuration(value)


@pytest.mark.parametrize(
    "value, flag",
    [
        ("value", False),
        (['DUMMY VALUE', 1, 1.4, False], False),
        ("value", True),
        ([['dummy', 'dummy2'], [1, 3], [1.4, 7.8], [False, True]], True),
    ],
)
def test_configuration_with_invalid_configurations(value, flag):
    with pytest.raises(TypeError):
        Configuration(value)


@pytest.mark.parametrize(
    "value",
    [
        [
            {'name': 'name1', 'value': 'value2'},
            {'name': 'name1', 'value': 'value2'},
            {'dummy_key': 'name3', 'value': 'value3'},
        ],
        [
            {'name': 'name1', 'value': 'value2'},
            {'name': 'name1', 'value': 'value2'},
            {'name': 'name1', 'dummy_value': 'value2'},
        ],
    ],
)
def test_configuration_with_incorrect_property_keys_values(value):
    with pytest.raises(TypeError):
        Configuration(value)


def test_configuration_compatibility_map():
    configuration = [{'name': 'config1', 'value': 'value1'}, {'name': 'config2', 'value': 'value1'}]
    compatibility_map = {
        'config1': {'name': 'new_config1', 'values': {'value1': 'new_value1', 'value2': 'new_value2'}},
        'config2': {'name': 'new_config2', 'values': {'value1': 'new_value1'}},
    }

    config = Configuration(
        configuration=configuration,
        property_key='name',
        property_value='value',
        compatibility_map=compatibility_map,
    )

    assert config['new_config1'] == 'new_value1'
    assert config['new_config2'] == 'new_value1'

    config['config1'] = 'value2'
    assert config['new_config1'] == 'new_value2'

    with pytest.raises(AttributeError):
        assert config['config1']
        assert config['config2']


def test_configuration_transformer_pipeline_aliases():
    configuration = [{'name': 'config1', 'value': 'value1'}, {'name': 'config2', 'value': 'value2'}]
    compatibility_map = {
        'config1': {'name': 'new_config1'},
        'config2': {'name': 'new_config2'},
    }

    config = Configuration(
        configuration=configuration,
        property_key='name',
        property_value='value',
        compatibility_map=compatibility_map,
    )

    assert config['new_config1'] == 'value1'
    assert config['new_config2'] == 'value2'

    config['config1'] = 'new_value2'
    assert config['new_config1'] == 'new_value2'

    with pytest.raises(AttributeError):
        assert config['config1']
        assert config['config2']


def test_transformer_backwards_compatibility_on_pipeline(mocker):
    # Test using values from constants.py
    pipeline_json = {
        'executorType': 'TRANSFORMER',
        'sdcVersion': '5.7.0',
        'pipelineId': 1,
        'commitId': None,
        'name': 'Test Pipeline',
        'version': 1,
    }

    configuration = [
        {'name': 'transformerEmrConnection.provisionNewCluster', 'value': True},
    ]
    pipeline_definition_json = {'configuration': configuration}

    library_definitions_json = json.dumps(
        {
            'pipeline': [
                {
                    'configDefinitions': [
                        {
                            'label': 'EMR Cluster',
                            'fieldName': 'emrClusterOption',
                            'name': 'transformerEmrConnection.emrClusterOption',
                        },
                    ]
                }
            ]
        }
    )

    builder = DummyPipelineBuilder()

    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter('always')
        pipeline = Pipeline(
            pipeline=pipeline_json,
            builder=builder,
            pipeline_definition=pipeline_definition_json,
            rules_definition={},
            library_definitions=library_definitions_json,
            control_hub=mocker.Mock(),
        )
        pipeline.fragment = False

        assert pipeline.configuration['transformerEmrConnection.emrClusterOption'] == 'PROVISION_CLUSTER'
        assert len(w) == 1
        assert issubclass(w[-1].category, DeprecationWarning)

        pipeline.configuration['transformerEmrConnection.provisionNewCluster'] = False
        assert len(w) == 2
        assert issubclass(w[-1].category, DeprecationWarning)
        assert pipeline.configuration['transformerEmrConnection.emrClusterOption'] == 'EXISTING_CLUSTER'



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2024

# fmt: off
from copy import deepcopy

import pytest

from streamsets.sdk import ControlHub
from streamsets.sdk.sch_api import ApiClient
from streamsets.sdk.sch_models import AzureEnvironment, SelfManagedDeployment

from .resources.sch_data import DUMMY_DEPLOYMENT_JSON, DUMMY_DEPLOYMENT_JSON_2

# fmt: on

VALID_INSTALL_SCRIPT = "Install Script"


class MockControlHub(ControlHub):
    def __init__(self, *args, **kwargs):
        self.api_client = MockApiClient()
        pass  # do not call super()

    def delete_deployment(self, *deployments):
        return

    def update_deployment(self, deployment):
        if (
            "labels" in deployment._data["engineConfiguration"]
            and deployment._data["engineConfiguration"]["labels"][0] == "Don't Update"
        ):
            return MockCommand(activation_error_data())
        super().update_deployment(deployment)


class MockResponse:
    def __init__(self, json_response):
        self.response = json_response

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response.encode()

    @property
    def text(self):
        return self.response


class MockCommand:
    def __init__(self, json_response):
        self._response = json_response

    @property
    def response(self):
        return MockResponse(self._response)


class MockApiClient(ApiClient):
    def __init__(self, *args, **kwargs):
        pass  # do not call super()

    def enable_deployments(self, deployment_ids):
        return True

    def create_deployment(self, data):
        return MockCommand(
            {
                "engineConfiguration": {
                    'advancedConfiguration': None,
                    'externalResourcesUri': None,
                    'maxCpuLoad': None,
                    'maxMemoryUsed': None,
                    'maxPipelinesRunning': None,
                    'jvmConfig': None,
                    'labels': None,
                    'stageLibs': None,
                    'scalaBinaryVersion': None,
                }
            }
        )

    def update_deployment(self, *args, **kwargs):
        return MockCommand(activation_error_data())

    def get_deployment(self, deployment_id):
        return MockCommand(activation_error_data())

    def enable_environments(self, environment_ids):
        return True

    def get_environment(self, environment_id):
        return MockCommand(activation_error_data())

    def update_environment(self, *args, **kwargs):
        return MockCommand(activation_error_data())

    def get_engine_version(self, engine_version_id):
        return MockCommand(
            {
                "id": "DC:5.10.0::RC3",
                "engineType": "DC",
                "engineVersion": "5.10.0",
                "creator": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                "createTime": 1711051750490,
                "lastModifiedBy": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                "lastModifiedOn": 1711051750490,
                "defaultJavaVersion": 8,
                "supportedJavaVersions": "8,17",
                'advancedConfiguration': None,
            }
        )

    def get_all_engine_versions(self, *args, **kwargs):
        return MockCommand(
            [
                {
                    "id": "DC:5.10.0::RC3",
                    "engineType": "DC",
                    "engineVersion": "5.9.0",
                    "creator": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                    "createTime": 1711051750490,
                    "lastModifiedBy": "a2ce9742-b78a-11eb-b93c-352da592f75a@admin",
                    "lastModifiedOn": 1711051750490,
                    "disabled": True,
                    "defaultJavaVersion": 8,
                    "supportedJavaVersions": "8,17",
                }
            ]
        )

    def get_self_managed_deployment_install_command(
        self, deployment_id, install_mechanism='DEFAULT', install_type=None, java_version=None
    ):
        return MockCommand(VALID_INSTALL_SCRIPT)


@pytest.fixture(scope="function")
def dummy_deployment_data():
    data = deepcopy(DUMMY_DEPLOYMENT_JSON)
    return data


@pytest.fixture(scope="function")
def dummy_deployment_data_2():
    data = deepcopy(DUMMY_DEPLOYMENT_JSON_2)
    return data


def activation_error_data():
    return {
        'stateDisplayLabel': 'ACTIVATION_ERROR',
        'status': 'ERROR',
        'statusDetail': 'Unexpected status CREATE_FAILED detected while checking deployment',
    }


def test_start_deployment_activation_error(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    with pytest.raises(RuntimeError):
        sch.start_deployment(deployment)


def test_update_deployment_activation_error(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    with pytest.raises(RuntimeError):
        sch.update_deployment(deployment)


def test_activate_environment_activation_error(azure_environment_json):
    sch = MockControlHub()
    environment = AzureEnvironment(azure_environment_json)
    environment._control_hub = sch
    with pytest.raises(RuntimeError):
        sch.activate_environment(environment)


def test_update_environment_activation_error(azure_environment_json):
    sch = MockControlHub()
    environment = AzureEnvironment(azure_environment_json)
    environment._control_hub = sch
    with pytest.raises(RuntimeError):
        sch.update_environment(environment)


def test_invalid_java_version_install_script(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    with pytest.raises(Exception):
        sch.get_self_managed_deployment_install_script(deployment, java_version='3')


def test_supported_java_version_install_script(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    assert sch.get_self_managed_deployment_install_script(deployment, java_version='17') == VALID_INSTALL_SCRIPT


def test_default_java_version_install_script(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    assert sch.get_self_managed_deployment_install_script(deployment) == VALID_INSTALL_SCRIPT


def test_broken_add_deployment(dummy_deployment_data):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data)
    deployment._control_hub = sch
    original_deployment_data = deployment._data
    with pytest.raises(KeyError):
        sch.add_deployment(deployment)
    assert deployment._data == original_deployment_data


def test_add_deployment_adds_disabled_engine(dummy_deployment_data_2):
    sch = MockControlHub()
    deployment = SelfManagedDeployment(dummy_deployment_data_2)
    deployment._control_hub = sch
    sch.add_deployment(deployment)



# Copyright 2022 StreamSets Inc.

# fmt: off
import uuid

from streamsets.sdk.sch_models import PipelineLabel
from streamsets.sdk.utils import SeekableList

# fmt: on


def test_create_pipeline_with_labels_logic(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    dev_data_generator = pipeline_builder.add_stage('Dev Data Generator')
    trash = pipeline_builder.add_stage('Trash')
    dev_data_generator >> trash
    pipeline_name = 'create_pipeline_with_label_{}'.format(str(uuid.uuid4()))
    label = str(uuid.uuid4())
    pipeline = pipeline_builder.build(title=pipeline_name, labels=[label])
    sch.publish_pipeline(pipeline)
    try:
        pipeline_labels = pipeline.labels
        assert len(pipeline_labels) == 1
        assert isinstance(pipeline_labels, SeekableList)
        assert isinstance(pipeline_labels[0], PipelineLabel)
        assert pipeline_labels[0].label == label
    finally:
        sch.delete_pipeline(pipeline)


def test_update_pipeline_with_labels_logic(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    dev_data_generator = pipeline_builder.add_stage('Dev Data Generator')
    trash = pipeline_builder.add_stage('Trash')
    dev_data_generator >> trash
    pipeline_name = 'update_pipeline_with_label_{}'.format(str(uuid.uuid4()))
    label = str(uuid.uuid4())
    pipeline = pipeline_builder.build(title=pipeline_name)
    sch.publish_pipeline(pipeline)
    pipeline.add_label(label)
    sch.publish_pipeline(pipeline)
    try:
        pipeline_labels = pipeline.labels
        assert len(pipeline_labels) == 1
        assert isinstance(pipeline_labels, SeekableList)
        assert isinstance(pipeline_labels[0], PipelineLabel)
        assert pipeline_labels[0].label == label
    finally:
        sch.delete_pipeline(pipeline)


def test_remove_labels(sch, sch_authoring_sdc_id):
    pipeline_builder = sch.get_pipeline_builder(engine_type='data_collector', engine_id=sch_authoring_sdc_id)
    dev_data_generator = pipeline_builder.add_stage('Dev Data Generator')
    trash = pipeline_builder.add_stage('Trash')
    dev_data_generator >> trash
    pipeline_name = 'create_pipeline_with_label_{}'.format(str(uuid.uuid4()))
    label1, label2 = str(uuid.uuid4()), str(uuid.uuid4())
    pipeline = pipeline_builder.build(title=pipeline_name, labels=[label1, label2])
    sch.publish_pipeline(pipeline)
    try:
        pipeline_labels = pipeline.labels
        assert len(pipeline_labels) == 2
        assert isinstance(pipeline_labels, SeekableList)
        assert isinstance(pipeline_labels[0], PipelineLabel)

        pipeline.remove_label(label1)
        sch.publish_pipeline(pipeline)

        pipeline_labels = pipeline.labels
        assert len(pipeline_labels) == 1
        assert isinstance(pipeline_labels, SeekableList)
        assert isinstance(pipeline_labels[0], PipelineLabel)
        assert pipeline_labels[0].label == label2
    finally:
        sch.delete_pipeline(pipeline)



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2024

# fmt: off
from copy import deepcopy

import pytest

from streamsets.sdk import ControlHub
from streamsets.sdk.sch_models import ScheduledTask

from .resources.scheduled_tasks_data import (
    DUMMY_CREATE_SCHEDULED_TASK_FAILURE_RESPONSE_JSON, DUMMY_INVALID_SCHEDULED_TASK_JSON,
)

# fmt: on


class MockControlHub(ControlHub):
    def __init__(self, *args, **kwargs):
        self.api_client = MockApiClient()
        pass


class MockApiClient:
    def __init__(self, *args, **kwargs):
        pass

    def create_scheduled_task(self, data, api_version=2):
        # return a JSON response for publishing a scheduled task, either for a valid task or an invalid task
        if 'INVALID' in data['data']['messages']:
            return MockCommand(dummy_create_scheduled_task_failure_response())


class MockCommand:
    def __init__(self, json_response):
        self._response = json_response

    @property
    def response(self):
        return MockResponse(self._response)


class MockResponse:
    def __init__(self, json_response):
        self.response = json_response

    def json(self):
        return self.response


def dummy_create_scheduled_task_failure_response():
    data = deepcopy(DUMMY_CREATE_SCHEDULED_TASK_FAILURE_RESPONSE_JSON)
    return data


@pytest.fixture(scope="function")
def dummy_invalid_scheduled_task_json():
    data = deepcopy(DUMMY_INVALID_SCHEDULED_TASK_JSON)
    return data


def test_add_scheduled_task_error_handling(dummy_invalid_scheduled_task_json):
    sch = MockControlHub()
    task = ScheduledTask(dummy_invalid_scheduled_task_json, control_hub=sch)
    with pytest.raises(Exception) as error_info:
        sch.add_scheduled_task(task)
        assert 'caused by' in str(error_info)
